#!/usr/bin/env python2
# -*- coding: utf-8 -*-
##################################################
# GNU Radio Python Flow Graph
# Title: Cfile To Zmq C
# Generated: Thu Dec 22 14:35:03 2016
##################################################

from gnuradio import blocks
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio import zeromq
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from optparse import OptionParser


class cfile_to_zmq_c(gr.top_block):

    def __init__(self, addr='tcp://127.0.0.1:12345', carrier_fc=0, filepath='', gain=0, repeat=False, samp_rate=32e3):
        gr.top_block.__init__(self, "Cfile To Zmq C")

        ##################################################
        # Parameters
        ##################################################
        self.addr = addr
        self.carrier_fc = carrier_fc
        self.filepath = filepath
        self.gain = gain
        self.repeat = repeat
        self.samp_rate = samp_rate

        ##################################################
        # Blocks
        ##################################################
        self.zeromq_push_sink_0 = zeromq.push_sink(gr.sizeof_gr_complex, 1, addr, 100, False, -1)
        self.blocks_throttle_0 = blocks.throttle(gr.sizeof_gr_complex*1, samp_rate,True)
        self.blocks_multiply_const_vxx_0 = blocks.multiply_const_vcc((10 ** (gain / 20.), ))
        self.blocks_file_source_0 = blocks.file_source(gr.sizeof_gr_complex*1, filepath, True)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.blocks_file_source_0, 0), (self.blocks_throttle_0, 0))    
        self.connect((self.blocks_multiply_const_vxx_0, 0), (self.zeromq_push_sink_0, 0))    
        self.connect((self.blocks_throttle_0, 0), (self.blocks_multiply_const_vxx_0, 0))    

    def get_addr(self):
        return self.addr

    def set_addr(self, addr):
        self.addr = addr

    def get_carrier_fc(self):
        return self.carrier_fc

    def set_carrier_fc(self, carrier_fc):
        self.carrier_fc = carrier_fc

    def get_filepath(self):
        return self.filepath

    def set_filepath(self, filepath):
        self.filepath = filepath
        self.blocks_file_source_0.open(self.filepath, True)

    def get_gain(self):
        return self.gain

    def set_gain(self, gain):
        self.gain = gain
        self.blocks_multiply_const_vxx_0.set_k((10 ** (self.gain / 20.), ))

    def get_repeat(self):
        return self.repeat

    def set_repeat(self, repeat):
        self.repeat = repeat

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.blocks_throttle_0.set_sample_rate(self.samp_rate)


def argument_parser():
    parser = OptionParser(usage="%prog: [options]", option_class=eng_option)
    parser.add_option(
        "", "--addr", dest="addr", type="string", default='tcp://127.0.0.1:12345',
        help="Set ZMQ address [default=%default]")
    parser.add_option(
        "", "--carrier-fc", dest="carrier_fc", type="eng_float", default=eng_notation.num_to_str(0),
        help="Set Carrier frequency (Hz) [default=%default]")
    parser.add_option(
        "", "--filepath", dest="filepath", type="string", default='',
        help="Set File name [default=%default]")
    parser.add_option(
        "", "--gain", dest="gain", type="eng_float", default=eng_notation.num_to_str(0),
        help="Set Gain (dB) [default=%default]")
    parser.add_option(
        "", "--samp-rate", dest="samp_rate", type="eng_float", default=eng_notation.num_to_str(32e3),
        help="Set Sample rate [default=%default]")
    return parser


def main(top_block_cls=cfile_to_zmq_c, options=None):
    if options is None:
        options, _ = argument_parser().parse_args()

    tb = top_block_cls(addr=options.addr, carrier_fc=options.carrier_fc, filepath=options.filepath, gain=options.gain, samp_rate=options.samp_rate)
    tb.start()
    try:
        raw_input('Press Enter to quit: ')
    except EOFError:
        pass
    tb.stop()
    tb.wait()


if __name__ == '__main__':
    main()
