from abc import ABCMeta,abstractmethod,abstractproperty

##
# @brief Lrit File Header abstract class.
# @author Miguel Dominguez
# @date 2016
class AbstractHeader(object):

   __metaclass__ = ABCMeta

   BYTE_SHIFT = 8

   ##
   # @fn headerIndex
   # @brief Return the header index of a header.
   # @details The header index should uniquely identify the type of header
   # in a given LRIT file.
   # @param[out] output The header index for the given header type
   @abstractmethod
   def headerIndex(self):
      pass

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   @abstractmethod
   def getHeader(self,data):
      pass

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   @abstractmethod
   def __str__(self):
      pass
