from AbstractHeader import AbstractHeader

##
# @brief NOAA LRIT header class.
# @details Identifies the product type
# @author Miguel Dominguez
# @date 2016
# @todo Create a function that parses product ID, product Sub ID, and parameter
# and returns the English-language interpretation
# @todo Further mask bits in the header.  Even though there are several 16-bit
# values they actually only contain a tiny number of legitimate values
class NOAALRITHeader(AbstractHeader):

   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 129
   @staticmethod
   def headerIndex():
      return 129

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.agencySignature = "".join(map(chr,data[3:7]))
      self.productId = (data[8] << self.BYTE_SHIFT) | data[9]
      self.productSubId = (data[10] << self.BYTE_SHIFT) | data[11]
      self.parameter = (data[12] << self.BYTE_SHIFT) | data[13]
      self.compression = data[14]
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "NOAA LRIT Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Agency signature: {0}\n".format(self.agencySignature)
      output += "Product ID: {0}\n".format(self.productId)
      output += "Product SubID: {0}\n".format(self.productSubId)
      output += "Parameter: {0}\n".format(self.parameter)
      output += "Compression: {0}\n".format(self.compression)
      return output
