from AbstractHeader import AbstractHeader
import datetime

##
# @brief Timestamp header class.
# @details For time-sensitive data.  The raw data counts days and milliseconds
# since January 1st, 1958.
# @author Miguel Dominguez
# @date 2016
class TimestampHeader(AbstractHeader):

   INITIAL_DATE = datetime.datetime(1958,1,1,0,0,0,0,None)
   
   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 5
   @staticmethod
   def headerIndex():
      return 5

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      dayDelta = (data[4] << self.BYTE_SHIFT) | data[5]
      msDelta = 0
      for i in range(4):
         msDelta <<= self.BYTE_SHIFT
         msDelta |= data[6 + i]
      timeDelta = datetime.timedelta(dayDelta,0,0,msDelta,0,0,0)
      self.timeStamp = self.INITIAL_DATE + timeDelta
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Timestamp Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Timestamp: {0}\n".format(self.timeStamp)
      return output
