#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 The Aerospace Corporation.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

"""
Algorithms

This module contains all the heavy algorithms used in this GUI.
"""

def read_LRIT_file(directory, file_name, stitch_images):
    image_file = os.path.join(directory, file_name)
    
    if stitch_images == False:
        image = _read_one_LRIT_file(image_file)
        name, _ = os.path.splitext(file_name)
        save_name = os.path.join('temp', name + '.jpg')
    else: # Perform stitching
        search_seg = image_file.index("seg")
        seg = int(image_file[search_seg+3:search_seg+6])
        image, name = _stitch_LRIT_files(directory, file_name, search_seg, seg)
        save_name = os.path.join('temp', file_name[:seg] + '.jpg')
    
    image.save(save_name,'JPEG')

def _stitch_LRIT_files(directory, file_name, search_seg, seg):
    '''Reads many LRIT files and provides image
    
    Inputs
    ======
    directory -- directory where file is located
    file_name -- name of base .lrit or .LRIT file
    search_seg -- 
    seg --
    
    Returns
    =======
    image -- stitched PIL Image object
    '''
    image_file = os.path.join(directory, file_name)
    lrit_images = [0]*1000
    lrit_images[seg-1] = _read_one_LRIT_file(image_file)
    
    files_start_with = file_name[:search_seg-len(directory)]
    files_end_with = file_name[image_file.index('dat'):-5]
    lrit_files = []
    for i in os.listdir(directory):
        if os.path.isfile(os.path.join(directory,i)) and files_start_with in i: # and files_end_with in i:
            lrit_files.append(i)
    
    seg_start = search_seg - len(directory) + 2
    seg_end = search_seg - len(directory) + 5
    
    start = seg # Regarding number in file, not index in lrit_images
    while 1: # Search for segments decreasing consecutively
        lrit_file = None
        file_found = False
        for f in lrit_files:
            if start - 1 == int(f[seg_start:seg_end]):
                start = start - 1
                lrit_file = f
                file_found = True
                break
        if file_found:
            lrit_images[start-1] = _read_one_LRIT_file(os.path.join(directory, lrit_file))
        else: # No more files can be found
            break
    
    end = seg
    while 1: # Search for segments increasing
        lrit_file = None
        file_found = False
        for f in lrit_files:
            if end + 1 == int(f[seg_start:seg_end]):
                end = end + 1
                lrit_file = f
                file_found = True
                break
        if file_found:
            lrit_images[end-1] = _read_one_LRIT_file(os.path.join(directory, lrit_file))
        else: # No more files can be found
            break
    
    image_width = lrit_images[seg-1].size[0]
    image_height = 0
    
    index = start - 1
    while index < end:
        image_height += lrit_images[index].size[1]
        index += 1
    
    image = Image.new('RGB', (image_width,image_height), None)
    height_pos = 0
    index = start - 1
    
    while index < end:
        height = lrit_images[index].size[1]
        image.paste(lrit_images[index], (0,height_pos))
        height_pos += height
        index += 1
    
    print lrit_files
    return image, files_start_with
    print lrit_files
    print start, seg, end
    print lrit_images[seg-1].size[1]
    print image_width, image_height

def _read_one_LRIT_file(image_file):
    '''Reads one LRIT file and provides image
    
    Inputs
    ======
    image_file -- full path to .lrit or .LRIT file
    
    Returns
    =======
    image -- PIL Image object
    '''
    with open(image_file, 'r') as f:
        file_data = np.fromfile(f,dtype='uint8')

        primary_header = []
        loc = 0
        for i in range(16):
            primary_header.append(file_data[loc])
            loc += 1
        total_header_bytes = (primary_header[4] << 24) + (primary_header[5] << 16) + (primary_header[6] << 8) + primary_header[7] - 16
    
        headers = []
        for i in xrange(total_header_bytes):
            try: a = file_data[loc]
            except:
                print "Error reading file headers for: ", image_file
                return
            headers.append(a)
            loc += 1
    
        image_header_found = False
        header_idx = 0
        while not image_header_found and header_idx < total_header_bytes:
            if headers[header_idx] == 1 and headers[header_idx+1] == 0 and headers[header_idx+2] == 9:
                image_header_found = True
                bits_per_pixel = headers[header_idx+3]
                image_width = (headers[header_idx+4] << 8) + headers[header_idx+5]
                image_height = (headers[header_idx+6] << 8) + headers[header_idx+7]
            else:
                offset = (headers[header_idx+1]<<8) + headers[header_idx + 2]
                header_idx = header_idx + offset
    
        if not image_header_found:
            print "No image structure record in ", image_file
            return
        
        bitmap_buffer = []
        for i in xrange(image_width*image_height):
            try: a = file_data[loc]
            except: 
                print "Error reading bitmap data in: ", image_file
                return
            bitmap_buffer.append(a)
            loc += 1
        
        image = Image.new('RGB', (image_width,image_height), 'black')
        
        pixels = image.load()
        for i in range(image.size[0]):
            for j in range(image.size[1]):
                pixels[i,j] = (bitmap_buffer[i + j*image_width],bitmap_buffer[i + j*image_width],bitmap_buffer[i + j*image_width])
        
        return image

if __name__ == "__main__":
    pass
    #read_LRIT_file('C:\LRITTEMP', 'gos13chnIR04rgnFDseg003res04dat088030155534.lrit', False)
    # _read_one_LRIT_file('C:\LRITTEMP', 'gos12chnIR04rgnSHseg005res04dat072232214863.lrit')
    
